#include <fstream>
#include <cstdlib>
#include <cmath>

const int N = 1000; // Nombre de pas de temps
const double dt  = 1./N; // Pas de temps

double uniforme(void)
{
  return (1.+rand())/(1.+RAND_MAX);
  // On renvoie un nombre aleatoire dans ]0,1]. Au numerateur, on ecrit 1.+rand() pour ne jamais renvoyer la valeur 0 (singularite du logarithme)
}


double gauss(void)
{
  // Methode de Box-Muller pour les Gaussiennes
  return sqrt( -2*log(uniforme()) )  *  cos( 2*M_PI*uniforme() );
}


int main(void)
{
  srand(2364); // On initialise le generateur avec une valeur quelconque.

  double t = 0, W = 0;
  // t va contenir le temps courant.
  // W va contenir la valeur du Brownien a l'instant t.

  std::ofstream sortie("brownien.dat");
  sortie << t << " " << W << "\n";

  for (int i=1; i<=N; i++)
	{
	  t = i*dt;
	  W += sqrt(dt) * gauss();
	  /*
		L'accroissement d'un Brownien sur un intervalle de
		 largeur dt a une variance de dt. C'est donc dt^1/2
		 fois une Gaussienne centree reduite.
	  */
	  
	  sortie << t << " " << W << "\n";	  
	}
  sortie.close();
  return 0;
}
