#include "entier.hpp"

entier::entier(unsigned n)
{
	if (n==0)
	  {
		nb_chiffres = 1;
		decimale = new int[1];
		decimale[0] = 0;
	  }
	else
	  {	
		nb_chiffres = 1 + (int)log10(n); // On compte le nombre de cases a reserver
		
		decimale = new int[nb_chiffres];
		for (unsigned k=0; k<nb_chiffres; k++)
		  {
			decimale[k] = n%10; // On cacule les chiffres de n en base 10, decimale[k] correspond au chiffre des 10^k
			n /= 10;
		  }
	  }
	
  }



// Le constructeur par copie
entier::entier(entier const& n)
{
  decimale = new int[n.nb_chiffres];
  nb_chiffres = n.nb_chiffres;
  
  for (unsigned i=0; i<nb_chiffres; i++)
	decimale[i] = n.decimale[i];
}




entier & entier::operator=(entier const& n)
{
  if (&n != this) //On verifie que l'on n'efface pas l'objet que l'on veut recopier (si on ecrivait n=n)
	{
	  if (nb_chiffres != n.nb_chiffres)
		{
		  delete decimale;
		  nb_chiffres = n.nb_chiffres;
		  decimale = new int[nb_chiffres];
		}
	  
	  for (unsigned i=0; i<nb_chiffres; i++)
		decimale[i] = n.decimale[i];
	}
  
  return *this;
}



entier operator+(entier const &n, entier const &m)
{
  // On effectue l'addition en calculant les chiffres un par un, "comme a l'ecole primaire"
  entier somme(0);


  // On calcule (on majore en fait) le nombre de chiffres de la somme (le plus grand des nombres de chiffres de n et de m, eventuellement plus un).
  somme.nb_chiffres = ((n.nb_chiffres < m.nb_chiffres) ? m : n).nb_chiffres;

  if (n.decimale[n.nb_chiffres-1] + m.decimale[m.nb_chiffres-1] >= 9)
	{
	  somme.nb_chiffres ++;
	}
  
  somme.decimale = new int[somme.nb_chiffres];
  
  int retenue=0;
  for (unsigned i=0; i<somme.nb_chiffres; i++)
	{
	  // Le i-eme chiffre de n et m est soit un element du tableau, soit 0. n_decimale designe le i-eme chiffre de n.
	  int n_decimale = (i<n.nb_chiffres) ? n.decimale[i] : 0 ;
	  int m_decimale = (i<m.nb_chiffres) ? m.decimale[i] : 0 ;
	  somme.decimale[i] = (n_decimale + m_decimale + retenue) % 10; // On fait la somme, en prenant en compte les eventuelles retenue.
	  retenue = (n_decimale + m_decimale + retenue)/10;
	}
  
  return somme;
}

entier& entier::operator+=(entier const &m)
{
  // On definit l'operateur += a partir de l'operateur + que l'on vient de definir
  *this = *this + m;
  return *this;
}




entier operator*(entier const &n, entier const &m)
{
  entier prod(0);


  // On majore le nombre de chiffre du produit
  prod.nb_chiffres = n.nb_chiffres + m.nb_chiffres - 1;

  if ((n.decimale[n.nb_chiffres-1]+1) * (m.decimale[m.nb_chiffres-1]+1) >= 10)
	{
	  prod.nb_chiffres ++;
	}
  
  prod.decimale = new int[prod.nb_chiffres];

  int retenue = 0;
  for (unsigned i=0 ; i<prod.nb_chiffres ; i++)
	{
	  int somme = 0;
	  for (unsigned j=0; j<=i; j++)
		{
		  // Aux retenues pres, le chiffre des 10^i est la somme des produits a*b ou a est le chiffre des 10^j de n et b est le chiffre des 10^(i-j) de m.
		  int n_decimale = (j < n.nb_chiffres) ? n.decimale[j] : 0 ;
		  int m_decimale = (i-j < m.nb_chiffres) ? m.decimale[i-j] : 0 ;
		  somme += n_decimale * m_decimale;
		}
	  somme += retenue;
	  prod.decimale[i] = somme % 10;
	  retenue = somme / 10;
	}
  
  return prod;
  
}
entier& entier::operator*=(entier const &m)
{
  // Ecriture de *= a partir de *
  *this = *this * m;
  return *this;
}

std::ostream& operator<<(std::ostream &o, entier const &n)
{
  for (int i=n.nb_chiffres-1; i>=0; i--) // On affiche les chiffres de n, en commencant par n.decimale[nb_chiffres-1]
	o << n.decimale[i];
  return o;
}
