#include "rationnel.hpp"

int pgcd(int a, int b)
// Calcule le pgcd de deux entiers non nuls.
{
  // On fait en sorte que a et b soient positifs (std::abs est la fonction valeur absolue, qui renvoie un entier si on lui fournit un entier)
  a = std::abs(a);
  b = std::abs(b);
  
  if (a<b) // On fait en sorte que b<a.
	{
	  int c = a;
	  a = b;
	  b = c;
	}

  int r;
  while (a%b != 0)
	{
	  r = a%b;
	  a = b;
	  b = r;	  
	}
  return b;
}


  
rationnel::rationnel(int n, int d)
{
  if (n==0) // Le rationnel 0/d vaut toujours 0/1
	  {
		num = 0;
		denom = 1;
	  }
  else
	{
	  int a = pgcd(n,d); // On va reduire la fraction (stocker le resultat de pgcd(n,d) permet de ne faire qu'une seule fois le calcul (et pas deux)
	  if (d>0) // On choisit de toujours avoir un denominateur positif, quitte a changer les deux signes
		{
		  num   = n/a;
		  denom = d/a;
		}
	  else
		{
		  num   = -n/a;
		  denom = -d/a;
		}
	}
}



// La somme de deux rationnels. L'appel de "rationnel(...,...)" est un appel du constructeur, qui va donc reduire la fraction
rationnel operator+(rationnel const &q1, rationnel const &q2)
{
  return rationnel(q1.num*q2.denom + q2.num*q1.denom, q1.denom*q2.denom);
}
rationnel& rationnel::operator+=(rationnel const &q)
{
  *this = *this + q; // + existe deja pour les rationnels
  return *this;
}

rationnel operator-(rationnel const &q1, rationnel const &q2)
{
  return q1 + (-q2); // a+b et -a existent deja pour les rationnels
}
rationnel& rationnel::operator-=(rationnel const &q)
{
  *this = *this - q;
  return *this;
}

// Le produit
rationnel operator*(rationnel const &q1, rationnel const &q2)
{
  return rationnel(q1.num*q2.num, q1.denom*q2.denom);
}
rationnel& rationnel::operator*=(rationnel const &q)
{
  *this = *this * q; // * existe deja
  return *this;
}

rationnel operator/(rationnel const &q1, rationnel const &q2)
{
  return rationnel(q1.num*q2.denom, q1.denom*q2.num);
}
rationnel& rationnel::operator/=(rationnel const &q)
{
  *this = *this / q; // "/" existe deja
  return *this;
}


std::ostream& operator<<(std::ostream& o, rationnel const &q)
{
  if (q.denom == 1) // Si le denominateur vaut 1, on ne l'affiche pas
	o << q.num;
  else
	o << q.num << "/" << q.denom;
  return o;
}
